@@ -1,3 +1,4 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
@@ -610,6 +611,7 @@ bus_ibus_impl_init (BusIBusImpl *ibus)
 #ifdef G_THREADS_ENABLED
     extern gint g_monitor_timeout;
     if (g_monitor_timeout != 0) {
+        /* Start the monitor of registry changes. */
         bus_registry_start_monitor_changes (ibus->registry);
     }
 #endif
@@ -810,13 +812,46 @@ _ibus_get_address (BusIBusImpl     *ibus,
     return reply;
 }
 
+
+static gboolean
+_timeout_cb (gpointer data)
+{
+    return TRUE;
+}
+
+static BusFactoryProxy *
+_get_factory_proxy(IBusEngineDesc *engine_desc)
+{
+    BusFactoryProxy *factory = NULL;
+
+    /* Add a timeout to wake up g_main_context_iteration in every 0.5 second,
+     * and then to check the factory assocated with the engine_desc */
+    guint timeout_id = g_timeout_add (500, _timeout_cb, NULL);
+
+    GTimer *timer = g_timer_new ();
+
+    /* Leave the loop, if it spends more than 5 seconds */
+    while (g_timer_elapsed (timer, NULL) <= 5.0) {
+        if (g_main_context_iteration (NULL, TRUE)) {
+            factory = bus_factory_proxy_get_from_engine (engine_desc);
+            if (factory != NULL) {
+                break;
+            }
+        }
+    }
+
+    g_source_remove (timeout_id);
+    g_timer_destroy (timer);
+
+    return factory;
+}
+
 static BusEngineProxy *
 bus_ibus_impl_create_engine (IBusEngineDesc *engine_desc)
 {
     IBusComponent *comp;
     BusFactoryProxy *factory;
     BusEngineProxy *engine;
-    GTimeVal t1, t2;
 
     factory = bus_factory_proxy_get_from_engine (engine_desc);
 
@@ -827,23 +862,8 @@ bus_ibus_impl_create_engine (IBusEngineDesc *engine_desc)
 
         if (!ibus_component_is_running (comp)) {
             ibus_component_start (comp, g_verbose);
-            g_get_current_time (&t1);
-            while (1) {
-                if (g_main_context_pending (NULL)) {
-                    g_main_context_iteration (NULL, FALSE);
-                    factory = bus_factory_proxy_get_from_engine (engine_desc);
-                    if (factory != NULL) {
-                        break;
-                    }
-                }
-                g_get_current_time (&t2);
-                if (t2.tv_sec - t1.tv_sec >= 5)
-                    break;
-            }
-        }
-        else {
-            factory = bus_factory_proxy_get_from_engine (engine_desc);
         }
+        factory = _get_factory_proxy (engine_desc);
     }
 
     if (factory == NULL) {
@@ -887,7 +907,6 @@ _context_request_engine_cb (BusInputContext *context,
                             BusIBusImpl     *ibus)
 {
     IBusEngineDesc *engine_desc = NULL;
-    BusEngineProxy *engine;
 
     /* context should has focus before request an engine */
     g_return_if_fail (bus_input_context_has_focus (context));
@@ -1921,8 +1940,9 @@ bus_ibus_impl_filter_keyboard_shortcuts (BusIBusImpl     *ibus,
 
         if (current_engine_desc != new_engine_desc) {
             bus_ibus_impl_set_context_engine_from_desc (ibus, context, new_engine_desc);
-            return TRUE;
         }
+
+        return TRUE;
     }
 
     return FALSE;
@@ -1934,7 +1954,8 @@ bus_ibus_impl_load_global_engine_name_from_config (BusIBusImpl *ibus)
     GValue value = { 0 };
     gchar *global_engine_name = NULL;
 
-    g_assert (IBUS_IS_CONFIG (ibus->config));
+    g_assert (BUS_IS_IBUS_IMPL (ibus));
+    g_return_val_if_fail (IBUS_IS_CONFIG (ibus->config), NULL);
 
     if (ibus_config_get_value (ibus->config, "general", "global_engine", &value) &&
         G_VALUE_TYPE (&value) == G_TYPE_STRING) {
@@ -1948,7 +1969,8 @@ bus_ibus_impl_load_global_engine_name_from_config (BusIBusImpl *ibus)
 static void
 bus_ibus_impl_save_global_engine_name_to_config (BusIBusImpl *ibus)
 {
-    g_assert (IBUS_IS_CONFIG (ibus->config));
+    g_assert (BUS_IS_IBUS_IMPL (ibus));
+    g_return_if_fail (IBUS_IS_CONFIG (ibus->config));
 
     if (ibus->use_global_engine && ibus->global_engine) {
         GValue value = { 0 };
@@ -1966,7 +1988,8 @@ bus_ibus_impl_load_global_previous_engine_name_from_config (BusIBusImpl *ibus)
     GValue value = { 0 };
     gchar *global_previous_engine_name = NULL;
 
-    g_assert (IBUS_IS_CONFIG (ibus->config));
+    g_assert (BUS_IS_IBUS_IMPL (ibus));
+    g_return_val_if_fail (IBUS_IS_CONFIG (ibus->config), NULL);
 
     if (ibus_config_get_value (ibus->config, "general", "global_previous_engine", &value) &&
         G_VALUE_TYPE (&value) == G_TYPE_STRING) {
@@ -1980,7 +2003,8 @@ bus_ibus_impl_load_global_previous_engine_name_from_config (BusIBusImpl *ibus)
 static void
 bus_ibus_impl_save_global_previous_engine_name_to_config (BusIBusImpl *ibus)
 {
-    g_assert (IBUS_IS_CONFIG (ibus->config));
+    g_assert (BUS_IS_IBUS_IMPL (ibus));
+    g_return_if_fail (IBUS_IS_CONFIG (ibus->config));
 
     if (ibus->use_global_engine && ibus->global_previous_engine_name) {
         GValue value = { 0 };
