@@ -1,7 +1,8 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,6 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#include <dbus/dbus.h>
 #include "ibuserror.h"
 
 IBusError *
@@ -61,7 +63,7 @@ ibus_error_new_from_printf (const gchar *name,
 }
 
 IBusError *
-ibus_error_new_from_message (DBusMessage *message)
+ibus_error_new_from_message (IBusMessage *message)
 {
     g_assert (message != NULL);
 
