@@ -1,7 +1,8 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,8 +22,6 @@
 #include "ibustext.h"
 
 /* functions prototype */
-static void         ibus_text_class_init   (IBusTextClass       *klass);
-static void         ibus_text_init         (IBusText            *text);
 static void         ibus_text_destroy      (IBusText            *text);
 static gboolean     ibus_text_serialize    (IBusText            *text,
                                             IBusMessageIter     *iter);
@@ -31,34 +30,7 @@ static gboolean     ibus_text_deserialize  (IBusText            *text,
 static gboolean     ibus_text_copy         (IBusText            *dest,
                                             const IBusText      *src);
 
-static IBusSerializableClass *parent_class = NULL;
-
-GType
-ibus_text_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusTextClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_text_class_init,
-        NULL,               /* class finialize */
-        NULL,               /* class data */
-        sizeof (IBusText),
-        0,
-        (GInstanceInitFunc) ibus_text_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_SERIALIZABLE,
-                                       "IBusText",
-                                       &type_info,
-                                       0);
-    }
-
-    return type;
-}
+G_DEFINE_TYPE (IBusText, ibus_text, IBUS_TYPE_SERIALIZABLE)
 
 static void
 ibus_text_class_init (IBusTextClass *klass)
@@ -66,7 +38,7 @@ ibus_text_class_init (IBusTextClass *klass)
     IBusObjectClass *object_class = IBUS_OBJECT_CLASS (klass);
     IBusSerializableClass *serializable_class = IBUS_SERIALIZABLE_CLASS (klass);
 
-    parent_class = (IBusSerializableClass *) g_type_class_peek_parent (klass);
+    ibus_text_parent_class = (IBusSerializableClass *) g_type_class_peek_parent (klass);
 
     object_class->destroy = (IBusObjectDestroyFunc) ibus_text_destroy;
 
@@ -98,7 +70,7 @@ ibus_text_destroy (IBusText *text)
         text->attrs = NULL;
     }
 
-    IBUS_OBJECT_CLASS (parent_class)->destroy ((IBusObject *)text);
+    IBUS_OBJECT_CLASS (ibus_text_parent_class)->destroy ((IBusObject *)text);
 }
 
 static gboolean
@@ -107,14 +79,17 @@ ibus_text_serialize (IBusText        *text,
 {
     gboolean retval;
 
-    retval = parent_class->serialize ((IBusSerializable *)text, iter);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_text_parent_class)->serialize (
+                        (IBusSerializable *)text, iter);
     g_return_val_if_fail (retval, FALSE);
 
     retval = ibus_message_iter_append (iter, G_TYPE_STRING, &text->text);
     g_return_val_if_fail (retval, FALSE);
 
-    if (text->attrs == NULL)
+    if (text->attrs == NULL) {
         text->attrs = ibus_attr_list_new ();
+        g_object_ref_sink (text->attrs);
+    }
 
     retval = ibus_message_iter_append (iter, IBUS_TYPE_ATTR_LIST, &text->attrs);
     g_return_val_if_fail (retval, FALSE);
@@ -129,7 +104,8 @@ ibus_text_deserialize (IBusText        *text,
     gboolean retval;
     gchar *str;
 
-    retval = parent_class->deserialize ((IBusSerializable *)text, iter);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_text_parent_class)->deserialize (
+                            (IBusSerializable *)text, iter);
     g_return_val_if_fail (retval, FALSE);
 
     retval = ibus_message_iter_get (iter, G_TYPE_STRING, &str);
@@ -145,6 +121,7 @@ ibus_text_deserialize (IBusText        *text,
     }
 
     retval = ibus_message_iter_get (iter, IBUS_TYPE_ATTR_LIST, &text->attrs);
+    g_object_ref_sink (text->attrs);
     g_return_val_if_fail (retval, FALSE);
     ibus_message_iter_next (iter);
 
@@ -157,8 +134,9 @@ ibus_text_copy (IBusText       *dest,
 {
     gboolean retval;
 
-    retval = parent_class->copy ((IBusSerializable *)dest,
-                                 (IBusSerializable *)src);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_text_parent_class)->copy (
+                            (IBusSerializable *)dest,
+                            (IBusSerializable *)src);
     g_return_val_if_fail (retval, FALSE);
 
     g_return_val_if_fail (IBUS_IS_TEXT (dest), FALSE);
@@ -166,8 +144,10 @@ ibus_text_copy (IBusText       *dest,
 
     dest->text = g_strdup (src->text);
     dest->is_static = FALSE;
-    if (src->attrs)
+    if (src->attrs) {
         dest->attrs = (IBusAttrList *)ibus_serializable_copy ((IBusSerializable *)src->attrs);
+        g_object_ref_sink (dest->attrs);
+    }
 
     return TRUE;
 }
@@ -293,7 +273,6 @@ ibus_text_append_attribute (IBusText *text,
 
     attr = ibus_attribute_new (type, value, start_index, end_index);
     ibus_attr_list_append (text->attrs, attr);
-    g_object_unref (attr);
 }
 
 guint
