@@ -1,3 +1,4 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -18,6 +19,7 @@ int main (int argc, char **argv)
 
 int main (int argc, char **argv)
 {
+#if 0
     gint fd;
     struct input_event e;
 
@@ -48,7 +50,7 @@ int main (int argc, char **argv)
     }
 
     g_object_unref (keymap);
-	return 0;
-
+#endif
+    return 0;
 }
 #endif
