@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright(c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright(c) 2007-2009 Red Hat, Inc.
+# Copyright(c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright(c) 2007-2010 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@ import gobject
 import pango
 import ibus
 from ibus._gtk import PangoAttrList
+from handle import Handle
 
 from gettext import dgettext
 _  = lambda a : dgettext("ibus", a)
@@ -74,7 +75,7 @@ class CandidateArea(gtk.HBox):
         self.__create_ui()
 
     def __create_ui(self):
-        if self.__orientation == gtk.ORIENTATION_VERTICAL:
+        if self.__orientation == ibus.ORIENTATION_VERTICAL:
             self.__vbox1 = gtk.VBox()
             self.__vbox1.set_homogeneous(True)
             self.__vbox2 = gtk.VBox()
@@ -92,7 +93,7 @@ class CandidateArea(gtk.HBox):
             label2.set_alignment(0.0, 0.5)
             label1.show()
 
-            if self.__orientation == gtk.ORIENTATION_VERTICAL:
+            if self.__orientation == ibus.ORIENTATION_VERTICAL:
                 label1.set_property("xpad", 8)
                 label2.set_property("xpad", 8)
                 ebox1 = EventBox()
@@ -161,14 +162,6 @@ class CandidateArea(gtk.HBox):
 
 class CandidatePanel(gtk.VBox):
     __gtype_name__ = "IBusCandidate"
-    __gproperties__ = {
-        'orientation' : (gtk.Orientation,        # type
-        'orientation of candidates',            # nick name
-        'the orientation of candidates list',    # description
-        0,
-        gobject.PARAM_READWRITE)                # flags
-        }
-
     __gsignals__ = {
         "cursor-up" : (
             gobject.SIGNAL_RUN_FIRST,
@@ -200,19 +193,23 @@ class CandidatePanel(gtk.VBox):
         self.__viewport = gtk.Viewport()
         self.__viewport.set_shadow_type(gtk.SHADOW_IN)
         self.__toplevel.add(self.__viewport)
-        self.__viewport.add(self)
+
+        hbox = gtk.HBox()
+        handle = Handle()
+        handle.connect("move-end", self.__handle_move_end_cb)
+        hbox.pack_start(handle)
+        hbox.pack_start(self)
+
+        self.__viewport.add(hbox)
         self.__toplevel.add_events(
             gdk.BUTTON_PRESS_MASK | \
             gdk.BUTTON_RELEASE_MASK | \
             gdk.BUTTON1_MOTION_MASK)
         self.__begin_move = False
-        # self.__toplevel.connect("button-press-event", self.__button_press_event_cb)
-        # self.__toplevel.connect("button-release-event", self.__button_release_event_cb)
-        # self.__toplevel.connect("motion-notify-event", self.__motion_notify_event_cb)
         self.__toplevel.connect("size-allocate", lambda w, a: self.__check_position())
 
-        self.__orientation = gtk.ORIENTATION_HORIZONTAL
-        self.__orientation = gtk.ORIENTATION_VERTICAL
+        self.__orientation = ibus.ORIENTATION_VERTICAL
+        self.__current_orientation = self.__orientation
         self.__preedit_visible = False
         self.__aux_string_visible = False
         self.__lookup_table_visible = False
@@ -222,10 +219,15 @@ class CandidatePanel(gtk.VBox):
         self.__aux_attrs = pango.AttrList()
         self.__lookup_table = None
 
-        self.__cursor_location = (0, 0)
+        self.__cursor_location = (0, 0, 0, 0)
+        self.__moved_cursor_location = None
 
         self.__recreate_ui()
 
+    def __handle_move_end_cb(self, handle):
+        # store moved location
+        self.__moved_cursor_location = self.__toplevel.get_position() + (self.__cursor_location[2], self.__cursor_location[3])
+
     def __recreate_ui(self):
         for w in self:
             self.remove(w)
@@ -249,10 +251,10 @@ class CandidatePanel(gtk.VBox):
             self.__aux_label.show()
 
         # create candidates area
-        self.__candidate_area = CandidateArea(self.__orientation)
+        self.__candidate_area = CandidateArea(self.__current_orientation)
         self.__candidate_area.set_no_show_all(True)
         self.__candidate_area.connect("candidate-clicked", lambda x, i, b, s: self.emit("candidate-clicked", i, b, s))
-        self.update_lookup_table(self.__lookup_table, self.__lookup_table_visible)
+        # self.update_lookup_table(self.__lookup_table, self.__lookup_table_visible)
 
         # create state label
         self.__state_label = Label()
@@ -272,7 +274,7 @@ class CandidatePanel(gtk.VBox):
         self.__pack_all_widgets()
 
     def __pack_all_widgets(self):
-        if self.__orientation == gtk.ORIENTATION_VERTICAL:
+        if self.__current_orientation == ibus.ORIENTATION_VERTICAL:
             # package all widgets in vertical mode
             image = gtk.Image()
             image.set_from_stock(gtk.STOCK_GO_UP, gtk.ICON_SIZE_MENU)
@@ -310,7 +312,7 @@ class CandidatePanel(gtk.VBox):
             vbox.pack_start(self.__aux_label, False, False, 0)
             self.pack_start(vbox, False, False, 5)
             self.pack_start(HSeparator(), False, False)
-            hbox= gtk.HBox()
+            hbox = gtk.HBox()
             hbox.pack_start(self.__candidate_area, True, True, 2)
             hbox.pack_start(VSeparator(), False, False)
             hbox.pack_start(self.__prev_button, False, False, 2)
@@ -382,18 +384,22 @@ class CandidatePanel(gtk.VBox):
                 )
 
     def update_lookup_table(self, lookup_table, visible):
-        if lookup_table == None:
-            lookup_table = ibus.LookupTable()
-
-        if visible:
-            self.show_lookup_table()
-        else:
+        # hide lookup table
+        if not visible:
             self.hide_lookup_table()
 
-        self.__lookup_table = lookup_table
+        self.__lookup_table = lookup_table or ibus.LookupTable()
+        orientation = self.__lookup_table.get_orientation()
+        if orientation not in (ibus.ORIENTATION_HORIZONTAL, ibus.ORIENTATION_VERTICAL):
+            orientation = self.__orientation
+        self.set_current_orientation(orientation)
         self.__refresh_candidates()
         self.__refresh_labels()
 
+        # show lookup table
+        if visible:
+            self.show_lookup_table()
+
     def show_lookup_table(self):
         self.__lookup_table_visible = True
         self.__candidate_area.set_no_show_all(False)
@@ -422,9 +428,12 @@ class CandidatePanel(gtk.VBox):
         self.__lookup_table.cursor_down()
         self.__refresh_candidates()
 
-    def set_cursor_location(self, x, y):
-        self.__cursor_location = (x, y)
-        self.__check_position()
+    def set_cursor_location(self, x, y, w, h):
+        # if cursor location is changed, we reset the moved cursor location
+        if self.__cursor_location != (x, y, w, h):
+            self.__cursor_location = (x, y, w, h)
+            self.__moved_cursor_location = None
+            self.__check_position()
 
     def __check_show_states(self):
         if self.__preedit_visible or \
@@ -442,28 +451,20 @@ class CandidatePanel(gtk.VBox):
         self.update_lookup_table(None, False)
         self.hide()
 
-    def set_orientation(self, orientation):
-        if self.__orientation == orientation:
+    def set_current_orientation(self, orientation):
+        if self.__current_orientation == orientation:
             return
-        self.__orientation = orientation
+        self.__current_orientation = orientation
         self.__recreate_ui()
         if self.__toplevel.flags() & gtk.VISIBLE:
             self.show_all()
 
-    def get_orientation(self):
-        return self.__orientation
+    def set_orientation(self, orientation):
+        self.__orientation = orientation
+        self.update_lookup_table(self.__lookup_table, self.__lookup_table_visible)
 
-    def do_set_property(self, property, value):
-        if property == 'orientation':
-            self.set_orientation(value)
-        else:
-            return gtk.DrawingArea.do_set_property(property, value)
-
-    def do_get_property(self, property):
-        if property == 'orientation':
-            return self.__orientation
-        else:
-            return gtk.DrawingArea.do_get_property(property)
+    def get_current_orientation(self):
+        return self.__current_orientation
 
     # def do_expose_event(self, event):
     #     self.style.paint_box(self.window,
@@ -482,58 +483,30 @@ class CandidatePanel(gtk.VBox):
         self.__toplevel.resize(1, 1)
 
     def __check_position(self):
-        bx = self.__cursor_location[0] + self.__toplevel.allocation.width
-        by = self.__cursor_location[1] + self.__toplevel.allocation.height
+        cursor_location = self.__moved_cursor_location or self.__cursor_location
+
+        cursor_right = cursor_location[0] + cursor_location[2]
+        cursor_bottom = cursor_location[1] + cursor_location[3]
+
+        window_right = cursor_right + self.__toplevel.allocation.width
+        window_bottom = cursor_bottom + self.__toplevel.allocation.height
 
         root_window = gdk.get_default_root_window()
         sx, sy = root_window.get_size()
 
-        if bx > sx:
+        if window_right > sx:
             x = sx - self.__toplevel.allocation.width
         else:
-            x = self.__cursor_location[0]
+            x = cursor_right
 
-        if by > sy:
-            y = sy - self.__toplevel.allocation.height
+        if window_bottom > sy:
+            # move the window just above the cursor so the window and a preedit string do not overlap.
+            y = cursor_location[1] - self.__toplevel.allocation.height
         else:
-            y = self.__cursor_location[1]
+            y = cursor_bottom
 
         self.move(x, y)
 
-    def __button_press_event_cb(self, widget, event):
-        if event.button == 1:
-            self.__begin_move = True
-            self.__press_pos = event.x_root, event.y_root
-            self.__toplevel.window.set_cursor(gdk.Cursor(gdk.FLEUR))
-            return True
-
-        if event.button == 3:
-            if self.get_orientation() == gtk.ORIENTATION_HORIZONTAL:
-                self.set_orientation(gtk.ORIENTATION_VERTICAL)
-            else:
-
-                self.set_orientation(gtk.ORIENTATION_HORIZONTAL)
-            return True
-        return False
-
-    def __button_release_event_cb(self, widget, event):
-        if event.button == 1:
-            del self.__press_pos
-            self.__begin_move = False
-            self.__toplevel.window.set_cursor(gdk.Cursor(gdk.LEFT_PTR))
-            return True
-        return False
-
-    def __motion_notify_event_cb(self, widget, event):
-        if self.__begin_move != True:
-            return False
-        x, y = self.__toplevel.get_position()
-        x  = int(x + event.x_root - self.__press_pos[0])
-        y  = int(y + event.y_root - self.__press_pos[1])
-        self.move(x, y)
-        self.__press_pos = event.x_root, event.y_root
-        return True
-
     def show_all(self):
         gtk.VBox.show_all(self)
         self.__toplevel.show_all()
